/***********************************************************************************************************************
 *
 * blueBill Mobile - open source birdwatching
 * ==========================================
 *
 * Copyright (C) 2009, 2010 by Tidalwave s.a.s. (http://www.tidalwave.it)
 * http://bluebill.tidalwave.it/mobile/
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * $Id: RdfJsonMarshaller.java,v 28b9ee5a69fa 2010/06/24 13:33:35 fabrizio $
 *
 **********************************************************************************************************************/
package it.tidalwave.semantic.io.json;

import javax.annotation.concurrent.ThreadSafe;
import javax.annotation.Nonnull;
import java.io.IOException;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.io.Writer;
import org.json.me.JSONException;
import org.json.me.JSONWriter;
import it.tidalwave.util.As;
import it.tidalwave.semantic.io.json.impl.MarshallerContextImpl;

/***********************************************************************************************************************
 *
 * See http://n2.talis.com/wiki/RDF_JSON_Specification
 *
 * @author  Fabrizio Giudici
 * @version $Id: $
 *
 **********************************************************************************************************************/
@ThreadSafe
public class RdfJsonMarshaller implements Marshaller
  {
    /*******************************************************************************************************************
     *
     * {@inheritDoc}
     *
     ******************************************************************************************************************/
    public void marshal (final @Nonnull OutputStream os, final @Nonnull As entity)
      throws IOException
      {
        marshal(new OutputStreamWriter(os, "UTF-8"), entity);
      }

    /*******************************************************************************************************************
     *
     * {@inheritDoc}
     *
     ******************************************************************************************************************/
    public void marshal (final @Nonnull Writer w, final @Nonnull As entity)
      throws IOException
      {
        try
          {
            marshal(new JSONWriter(w), entity);
            w.flush();
          }
        catch (JSONException e)
          {
            throw new IOException(e.toString()); // Java 5 / Android compatibility
          }
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    private void marshal (final @Nonnull JSONWriter jw, final @Nonnull As entity)
      throws IOException, JSONException
      {
        jw.object();
        final MarshallerContextImpl context = new MarshallerContextImpl(entity);
        context.marshalPendingEntities(jw);
        jw.endObject();
      }
  }
