/***********************************************************************************************************************
 *
 * blueBill Mobile - open source birdwatching
 * ==========================================
 *
 * Copyright (C) 2009, 2010 by Tidalwave s.a.s. (http://www.tidalwave.it)
 * http://bluebill.tidalwave.it/mobile/
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * $Id: RdfJsonUnmarshaller.java,v d454be68ee92 2010/06/24 22:24:13 fabrizio $
 *
 **********************************************************************************************************************/
package it.tidalwave.semantic.io.json;

import javax.annotation.Nonnull;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.Reader;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.List;
import org.json.me.JSONArray;
import org.json.me.JSONException;
import org.json.me.JSONObject;
import org.json.me.JSONTokener;
import it.tidalwave.util.As;
import it.tidalwave.util.Id;
import it.tidalwave.semantic.io.Unmarshaller;
import it.tidalwave.semantic.io.json.impl.DehydratedTriple;
import it.tidalwave.semantic.io.json.impl.UnmarshallerContextImpl;
import it.tidalwave.semantic.io.json.impl.Literals;
import it.tidalwave.semantic.io.json.impl.Vocabulary;

/***********************************************************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id: $
 *
 **********************************************************************************************************************/
public class RdfJsonUnmarshaller implements Unmarshaller
  {
    /*******************************************************************************************************************
     *
     * {@inheritDoc}
     *
     ******************************************************************************************************************/
    @Nonnull
    public <T extends As> T unmarshal (final @Nonnull InputStream is, @Nonnull Class<T> clazz)
      throws IOException
      {
        return unmarshal(new InputStreamReader(is, "UTF-8"), clazz);
      }

    /*******************************************************************************************************************
     *
     * {@inheritDoc}
     *
     ******************************************************************************************************************/
    @Nonnull
    public <T extends As> T unmarshal (final @Nonnull Reader r, @Nonnull Class<T> clazz)
      throws IOException
      {
        try
          {
            return unmarshal(new JSONTokener(readString(r)), clazz);
          }
        catch (JSONException e)
          {
            throw new IOException(e.toString()); // Java 5 / Android compatibility
          }
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Nonnull
    public <T extends As> T unmarshal (final @Nonnull JSONTokener jt, @Nonnull Class<T> clazz)
      throws IOException, JSONException
      {
        final JSONObject rootObject = (JSONObject)jt.nextValue();
        // FIXME: infer the type from the class - maybe with annotations?
        final Id primaryPredicate = new Id("http://www.tidalwave.it/rdf/observation/2010/06/22#ObservationSet");
        final UnmarshallerContextImpl context = new UnmarshallerContextImpl(unmarshalObject(rootObject));
        
        for (final DehydratedTriple triple : context.findTriples(Vocabulary.RDF_ABOUT, primaryPredicate))
          {
            return clazz.cast(context.find(triple.getSubject()));
          }

        throw new IOException("Can't find an object of type " + clazz);
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Nonnull
    private List<DehydratedTriple> unmarshalObject (final @Nonnull JSONObject parentObject)
      throws JSONException
      {
        final List<DehydratedTriple> triples = new ArrayList<DehydratedTriple>();
        
        for (final Enumeration<String> e = parentObject.keys(); e.hasMoreElements();)
          {
            final String subject = e.nextElement();
            final JSONObject object1 = parentObject.getJSONObject(subject);

            for (Enumeration<String> e2 = object1.keys(); e2.hasMoreElements(); )
              {
                final String predicate = e2.nextElement();
                final JSONArray objects = object1.getJSONArray(predicate);

                for (int i = 0; i < objects.length(); i++)
                  {
                    final Object object = Literals.unmarshalLiteral(objects.getJSONObject(i));
                    final DehydratedTriple triple = new DehydratedTriple(new Id(subject), new Id(predicate), object);
                    triples.add(triple);
                  }
              }
          }

        return triples;
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Nonnull
    private static String readString (final @Nonnull Reader r)
      throws IOException
      {
        final StringBuilder buffer = new StringBuilder();

        for (;;)
          {
            int c = r.read();

            if (c < 0)
              {
                break;
              }

            buffer.append((char)c);
          }

        return buffer.toString();
      }
  }
