/***********************************************************************************************************************
 *
 * blueBill Mobile - open source birdwatching
 * ==========================================
 *
 * Copyright (C) 2009, 2010 by Tidalwave s.a.s. (http://www.tidalwave.it)
 * http://bluebill.tidalwave.it/mobile/
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * $Id: AbstractTriple.java,v 96a070c6a0a0 2010/06/23 23:07:19 fabrizio $
 *
 **********************************************************************************************************************/
package it.tidalwave.semantic.io.json.impl;

import javax.annotation.CheckForNull;
import javax.annotation.Nonnull;
import it.tidalwave.util.Id;

/***********************************************************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id: $
 *
 **********************************************************************************************************************/
public class AbstractTriple<T>
  {
    @Nonnull
    private final T subject;

    @Nonnull
    private final Id predicate;

    @Nonnull
    private final Object object;

    public AbstractTriple (final @Nonnull T subject, final @Nonnull Id predicate, final @Nonnull Object object)
      {
        this.subject = subject;
        this.predicate = predicate;
        this.object = object;
      }

    @Nonnull
    public T getSubject()
      {
        return subject;
      }

    @Nonnull
    public Id getPredicate()
      {
        return predicate;
      }
    
    @Nonnull
    public Object getObject()
      {
        return object;
      }

    @Override
    public boolean equals (final @CheckForNull Object object)
      {
        if (object == null)
          {
            return false;
          }

        if (getClass() != object.getClass())
          {
            return false;
          }

        final AbstractTriple<T> other = (AbstractTriple<T>)object;

        return (this.subject.equals(other.subject)) &&
               (this.predicate.equals(other.predicate)) &&
               (this.object.equals(other.object));
      }

    @Override
    public int hashCode()
      {
        int hash = 7;
        hash = 37 * hash + subject.hashCode();
        hash = 37 * hash + predicate.hashCode();
        hash = 37 * hash + object.hashCode();
        return hash;
      }

    @Override @Nonnull
    public String toString()
      {
        return String.format("<%s> <%s> <%s>", subject, predicate, object);
      }
  }
