/***********************************************************************************************************************
 *
 * blueBill Mobile - open source birdwatching
 * ==========================================
 *
 * Copyright (C) 2009, 2010 by Tidalwave s.a.s. (http://www.tidalwave.it)
 * http://bluebill.tidalwave.it/mobile/
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * $Id: UnmarshallerContextImpl.java,v 021c10c2ba5b 2010/06/24 22:23:50 fabrizio $
 *
 **********************************************************************************************************************/
package it.tidalwave.semantic.io.json.impl;

import javax.annotation.Nonnull;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.SortedMap;
import java.util.Stack;
import java.util.TreeMap;
import it.tidalwave.util.As;
import it.tidalwave.util.Id;
import it.tidalwave.util.NotFoundException;
import it.tidalwave.netbeans.util.Locator;
import it.tidalwave.semantic.io.json.spi.TripleUnmarshaller;
import it.tidalwave.semantic.io.json.spi.TripleUnmarshaller.Context;
import it.tidalwave.semantic.io.json.spi.TripleUnmarshallerFactory;

/***********************************************************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id: $
 *
 **********************************************************************************************************************/
public class UnmarshallerContextImpl implements Context
  {
    private final List<DehydratedTriple> triples;

    private final Map<Id, Object> entities = new HashMap<Id, Object>();

    private final SortedMap<Id, List<DehydratedTriple>> trimpleMapBySubject;

    private final Stack<Object> stack = new Stack<Object>();

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    public UnmarshallerContextImpl (final @Nonnull List<DehydratedTriple> triples)
      {
        this.triples = triples;
        trimpleMapBySubject = groupedBySubject(triples);
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Nonnull
    public As unmarshal (final @Nonnull Id id)
      {
        final List<DehydratedTriple> subjectTriples = trimpleMapBySubject.get(id);
        final TripleUnmarshaller entityFactory = findEntityFactory(subjectTriples);
        final As result = entityFactory.unmarshal(subjectTriples, this);
        entities.put(id, result);

        return result;
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Nonnull
    public Object find (final @Nonnull Object object)
      {
        Object result = null;
        Id id = null;

        if (object instanceof Id)
          {
            id = (Id)object;
          }
        else if (object instanceof Reference)
          {
            id = ((Reference)object).getId();
          }
        else
          {
            throw new IllegalArgumentException("Only accept an Id or a Reference");
          }

        result = entities.get(id);

        if (result == null)
          {
            result = unmarshal(id);
          }

        return result;
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Nonnull
    private TripleUnmarshaller findEntityFactory (final @Nonnull List<DehydratedTriple> triples)
      {
        final TripleUnmarshallerFactory unmarshallerFactory = Locator.find(TripleUnmarshallerFactory.class);
        try
          {
            return unmarshallerFactory.findTripleUnmarshallerFor(triples);
          }
        catch (NotFoundException e)
          {
            throw new RuntimeException(e);
          }
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Nonnull
    private static SortedMap<Id, List<DehydratedTriple>> groupedBySubject (final @Nonnull List<DehydratedTriple> triples)
      {
        final SortedMap<Id, List<DehydratedTriple>> result = new TreeMap<Id, List<DehydratedTriple>>();

        for (final DehydratedTriple triple : triples)
          {
            final Id subject = triple.getSubject();
            List<DehydratedTriple> subTriples = result.get(subject);

            if (subTriples == null)
              {
                subTriples = new ArrayList<DehydratedTriple>();
                result.put(subject, subTriples);
              }

            subTriples.add(triple);
          }

        return result;
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    public void push (final @Nonnull Object object)
      {
        stack.push(object);
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    public void pop()
      {
        stack.pop();
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Nonnull
    public Object peek()
      {
        return stack.peek();
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Nonnull
    public Collection<DehydratedTriple> findTriples (final @Nonnull Id predicate, final @Nonnull Id objectId)
      {
        final List<DehydratedTriple> result = new ArrayList<DehydratedTriple>();

        for (final DehydratedTriple triple : triples)
          {
            if (triple.getPredicate().equals(predicate))
              {
                if (matches(objectId, triple.getObject()))
                  {
                    result.add(triple);
                  }
              }
          }

        return result;
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    public boolean matches (final @Nonnull Id objectId, final @Nonnull Object object)
      {
        if (object instanceof Reference)
          {
            if (((Reference)object).getId().equals(objectId))
              {
                return true;
              }
          }
        else if (object instanceof Id)
          {
            if (object.equals(objectId))
              {
                return true;
              }
          }
        else if (object instanceof String)
          {
            if (object.equals(objectId.stringValue()))
              {
                return true;
              }
          }

        return false;
      }
  }
