/***********************************************************************************************************************
 *
 * blueBill Mobile - open source birdwatching
 * ==========================================
 *
 * Copyright (C) 2009, 2010 by Tidalwave s.a.s. (http://www.tidalwave.it)
 * http://bluebill.tidalwave.it/mobile/
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * $Id: TripleUnmarshallerSupport.java,v 96a070c6a0a0 2010/06/23 23:07:19 fabrizio $
 *
 **********************************************************************************************************************/
package it.tidalwave.semantic.io.json.spi;

import javax.annotation.CheckForNull;
import javax.annotation.Nonnull;
import java.util.ArrayList;
import java.util.List;
import it.tidalwave.util.Id;
import it.tidalwave.util.NotFoundException;
import it.tidalwave.semantic.io.json.impl.DehydratedTriple;

/***********************************************************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id: $
 *
 **********************************************************************************************************************/
public abstract class TripleUnmarshallerSupport implements TripleUnmarshaller
  {
    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Nonnull
    protected DehydratedTriple findTripleWithPredicate (final @Nonnull List<DehydratedTriple> triples,
                                                        final @Nonnull Id predicate)
      throws NotFoundException
      {
        return NotFoundException.throwWhenEmpty(findTriplesWithPredicate(triples, predicate),
                                                "No triples with predicate %s in subject %s",
                                                predicate, triples.get(0).getSubject()).
                                                get(0);
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Nonnull
    protected List<DehydratedTriple> findTriplesWithPredicate (final @Nonnull List<DehydratedTriple> triples,
                                                               final @Nonnull Id predicate)
      {
        final List<DehydratedTriple> result = new ArrayList<DehydratedTriple>();

        for (final DehydratedTriple triple : triples)
          {
            if (triple.getPredicate().equals(predicate))
              {
                result.add(triple);
              }
          }

        return result;
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @CheckForNull
    protected Object getLiteral (final List<DehydratedTriple> triples, final @Nonnull Id predicate)
      {
        try
          {
            return findTripleWithPredicate(triples, predicate).getObject();
          }
        catch (NotFoundException e)
          {
            return null;
          }
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @CheckForNull
    protected Object findEntity (final Context context, final List<DehydratedTriple> triples, final @Nonnull Id predicate)
      {
        try
          {
            return context.find(findTripleWithPredicate(triples, predicate).getObject());
          }
        catch (NotFoundException e)
          {
            return null;
          }
      }
  }
